==> unicore-gateway6-4.3.0-2.src.tar.gz: src/main/java/eu/unicore/gateway/VSite.java <==
/**
 * Copyright (c) 2005, Forschungszentrum Juelich
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met: 
 * 
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the Forschungszentrum Juelich nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package eu.unicore.gateway;

import java.io.IOException;
import java.net.ConnectException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.UnknownHostException;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.net.ssl.SSLSocket;

import org.apache.commons.httpclient.HttpClient;
import org.apache.log4j.Logger;

import eu.unicore.gateway.client.HttpClientFactory;
import eu.unicore.gateway.util.LogUtil;
import eu.unicore.gateway.util.XURI;

public class VSite implements Site {

	private static final Logger log = LogUtil.getLogger(LogUtil.GATEWAY,VSite.class);

	private static final ExecutorService pingService=new ThreadPoolExecutor(2, 10,
			60L, TimeUnit.SECONDS,
			new SynchronousQueue<Runnable>());

	private final String name;
	private final URI virtualURI;
	private final int index;
	private final int offset;
	private final boolean isSecure;
	private final URI realURI;
	private final InetAddress inetaddress;
	private final HttpClientFactory clientFactory;
	
	//cached client
	private HttpClient client;

	//for monitoring
	private int numberOfRequests;
	private String errorMessage="OK";

	private Boolean isUp=null;
	
	/**
	 * @param name
	 * @param uri 
	 * @throws UnknownHostException 
	 */
	public VSite(URI gatewayURI, String name, String uri, HttpClientFactory clientFactory) throws UnknownHostException, URISyntaxException{
		if(uri==null)throw new IllegalArgumentException("URI can't be null.");
		URI realuri=new URI(uri);
		this.realURI=realuri;
		this.inetaddress = InetAddress.getByName(getRealURI().getHost());
		if(name==null || name.trim().length()==0)throw new IllegalArgumentException("VSite needs a name.");
		this.name = name;
		this.virtualURI = new URI(gatewayURI+"/"+name);
		index=new XURI(gatewayURI).countPathElements();
		offset=name.length()+2;
		isSecure="HTTPS".equalsIgnoreCase(realuri.getScheme());
		this.clientFactory = clientFactory;
		log.info("new virtual site: <"+virtualURI.toString()+"> at <"+uri+">" );
		
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#getName()
	 */
	public String getName()
	{
		return name;
	}

	public URI getVirtualURI()
	{
		return virtualURI;
	}
	
	public URI getRealURI()
	{
		return realURI;
	}
	
	public HttpClient getClient()
	{
		return client;
	}

	public void setClient(HttpClient client)
	{
		this.client = client;
	}

	/**
	 * return the real internet address of the site
	 */
	public InetAddress getInetAddress() 
	{
		return this.inetaddress;
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#getNumberOfRequests()
	 */
	public int getNumberOfRequests()
	{
		return numberOfRequests;
	}  

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#getStatusMessage()
	 */
	public String getStatusMessage(){
		return errorMessage;
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#ping()
	 */
	public boolean ping()
	{
		return ping(2000);
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#ping(int)
	 */
	public boolean ping(int timeout)
	{
		Future<Boolean> res=pingService.submit(new Callable<Boolean>(){
			public Boolean call(){
				Socket s=null;
				try{
					if(isSecure){
						s=clientFactory.getSSLSocketFactory().createSocket(getRealURI().getHost(), getRealURI().getPort());
						((SSLSocket)s).getSession();
					}else{
						s=new Socket(getInetAddress(), getRealURI().getPort());
					}
					errorMessage="OK";
					if(isUp==null || !isUp){
						log.info("VSite '"+name+"' @ "+getRealURI()+" is up.");
						isUp=true;
					}
					return Boolean.TRUE;
				}catch(ConnectException ce){
					if(isUp==null || isUp){
						log.info("VSite '"+name+"' @ "+getRealURI()+" is down.");
						isUp=false;
					}
					errorMessage="Site is down: connection refused.";
				}catch(Exception e){
					LogUtil.logException("Error pinging site",e,log);
					errorMessage="Site unavailable: "+LogUtil.getRootCause(e);
				}
				finally{
					if(s!=null)try{
						s.close();
					}catch(IOException ioe){
						LogUtil.logException("Error closing socket", ioe,log);
					}
				}
				return Boolean.FALSE;
			}
		});
		try{
			Boolean result=res.get(timeout, TimeUnit.MILLISECONDS);
			return result;
		}catch(Exception tex){
			LogUtil.logException("Error waiting for ping result", tex, log);
			errorMessage = "Timeout";
		}
		return false;
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#resolve(java.lang.String)
	 */
	public String resolve(String uri)
	{   
		int start=uri.toLowerCase().indexOf("/"+name.toLowerCase()+"/")+offset;
		String endpart=uri.substring(start);
		if (endpart.trim().length() == 0)
			return getRealURI().toString();
		else return getRealURI() + "/" + endpart;
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#accept(java.lang.String)
	 */
	public boolean accept(String uri)
	{
		try
		{
			XURI xuri = new XURI(new URI(uri));
			String sitename = xuri.getPathElement(index);
			boolean decision = (sitename != null && sitename.equalsIgnoreCase(getName()));
			if(log.isTraceEnabled()){
				log.trace("is " + uri + " a correct URI for this (" + this.toString() + ")Vsite ? " + decision);
			}
			if(decision)numberOfRequests++;
			return decision;
		}
		catch (URISyntaxException e)
		{
			LogUtil.logException("cannot parse the uri, " + uri + ". this must happen before attempting to map to a registered vsite",e);
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#select(java.lang.String)
	 */
	public final VSite select(String clientIP){
		return this;
	}
	
	public String toString()
	{
		return "VSite ::: name=" + getName() + ", virtualuri=" + getVirtualURI() + ", " + "realuri="+getRealURI().toString()+", inetaddress="+getInetAddress();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + ((realURI == null) ? 0 : realURI.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		VSite other = (VSite) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (realURI == null) {
			if (other.realURI != null)
				return false;
		} else if (!realURI.equals(other.realURI))
			return false;
		return true;
	}

	

}

==> unicore-gateway6-5.0.0-0.src.tar.gz: src/main/java/eu/unicore/gateway/VSite.java <==
/**
 * Copyright (c) 2005, Forschungszentrum Juelich
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are
 * permitted provided that the following conditions are met: 
 * 
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the Forschungszentrum Juelich nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package eu.unicore.gateway;

import java.io.IOException;
import java.net.ConnectException;
import java.net.InetAddress;
import java.net.Socket;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.UnknownHostException;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;

import org.apache.commons.httpclient.HttpClient;
import org.apache.log4j.Logger;

import eu.emi.security.authn.x509.impl.SocketFactoryCreator;
import eu.unicore.gateway.util.LogUtil;
import eu.unicore.gateway.util.XURI;
import eu.unicore.security.canl.AuthnAndTrustProperties;
import eu.unicore.util.Log;

public class VSite implements Site {

	private static final Logger log = Log.getLogger(LogUtil.GATEWAY,VSite.class);

	private static final ExecutorService pingService=new ThreadPoolExecutor(2, 10,
			60L, TimeUnit.SECONDS,
			new SynchronousQueue<Runnable>());

	private final String name;
	private final URI virtualURI;
	private final int index;
	private final int offset;
	private final boolean isSecure;
	private final URI realURI;
	private final InetAddress inetaddress;
	private final AuthnAndTrustProperties securityCfg;
	
	//cached client
	private HttpClient client;

	//for monitoring
	private int numberOfRequests;
	private String errorMessage="OK";

	private Boolean isUp=null;
	
	/**
	 * @param name
	 * @param uri 
	 * @throws UnknownHostException 
	 */
	public VSite(URI gatewayURI, String name, String uri, AuthnAndTrustProperties securityCfg) throws UnknownHostException, URISyntaxException{
		if(uri==null)throw new IllegalArgumentException("URI can't be null.");
		URI realuri=new URI(uri);
		this.realURI=realuri;
		this.inetaddress = InetAddress.getByName(getRealURI().getHost());
		if(name==null || name.trim().length()==0)throw new IllegalArgumentException("VSite needs a name.");
		this.name = name;
		this.virtualURI = new URI(gatewayURI+"/"+name);
		index=new XURI(gatewayURI).countPathElements();
		offset=name.length()+2;
		isSecure="HTTPS".equalsIgnoreCase(realuri.getScheme());
		this.securityCfg = securityCfg;
		log.info("new virtual site: <"+virtualURI.toString()+"> at <"+uri+">" );
		
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#getName()
	 */
	public String getName()
	{
		return name;
	}

	public URI getVirtualURI()
	{
		return virtualURI;
	}
	
	public URI getRealURI()
	{
		return realURI;
	}
	
	public HttpClient getClient()
	{
		return client;
	}

	public void setClient(HttpClient client)
	{
		this.client = client;
	}

	/**
	 * return the real internet address of the site
	 */
	public InetAddress getInetAddress() 
	{
		return this.inetaddress;
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#getNumberOfRequests()
	 */
	public int getNumberOfRequests()
	{
		return numberOfRequests;
	}  

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#getStatusMessage()
	 */
	public String getStatusMessage(){
		return errorMessage;
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#ping()
	 */
	public boolean ping()
	{
		return ping(2000);
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#ping(int)
	 */
	public boolean ping(int timeout)
	{
		Future<Boolean> res=pingService.submit(new Callable<Boolean>(){
			public Boolean call(){
				Socket s=null;
				try{
					if(isSecure){
						SSLSocketFactory factory = SocketFactoryCreator.getSocketFactory(
							securityCfg.getCredential(), securityCfg.getValidator()); 
						s=factory.createSocket(getRealURI().getHost(), getRealURI().getPort());
						((SSLSocket)s).getSession();
					}else{
						s=new Socket(getInetAddress(), getRealURI().getPort());
					}
					errorMessage="OK";
					if(isUp==null || !isUp){
						log.info("VSite '"+name+"' @ "+getRealURI()+" is up.");
						isUp=true;
					}
					return Boolean.TRUE;
				}catch(ConnectException ce){
					if(isUp==null || isUp){
						log.info("VSite '"+name+"' @ "+getRealURI()+" is down.");
						isUp=false;
					}
					errorMessage="Site is down: connection refused.";
				}catch(Exception e){
					LogUtil.logException("Error pinging site",e,log);
					errorMessage="Site unavailable: " + Log.getDetailMessage(e);
				}
				finally{
					if(s!=null)try{
						s.close();
					}catch(IOException ioe){
						LogUtil.logException("Error closing socket", ioe,log);
					}
				}
				return Boolean.FALSE;
			}
		});
		try{
			Boolean result=res.get(timeout, TimeUnit.MILLISECONDS);
			return result;
		}catch(Exception tex){
			LogUtil.logException("Error waiting for ping result", tex, log);
			errorMessage = "Timeout";
		}
		return false;
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#resolve(java.lang.String)
	 */
	public String resolve(String uri)
	{   
		int start=uri.toLowerCase().indexOf("/"+name.toLowerCase()+"/")+offset;
		String endpart=uri.substring(start);
		if (endpart.trim().length() == 0)
			return getRealURI().toString();
		else return getRealURI() + "/" + endpart;
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#accept(java.lang.String)
	 */
	public boolean accept(String uri)
	{
		try
		{
			XURI xuri = new XURI(new URI(uri));
			String sitename = xuri.getPathElement(index);
			boolean decision = (sitename != null && sitename.equalsIgnoreCase(getName()));
			if(log.isTraceEnabled()){
				log.trace("is " + uri + " a correct URI for this (" + this.toString() + ")Vsite ? " + decision);
			}
			if(decision)numberOfRequests++;
			return decision;
		}
		catch (URISyntaxException e)
		{
			LogUtil.logException("cannot parse the uri, " + uri + ". this must happen before attempting to map to a registered vsite",e);
			return false;
		}
	}

	/* (non-Javadoc)
	 * @see eu.unicore.gateway.Site#select(java.lang.String)
	 */
	public final VSite select(String clientIP){
		return this;
	}
	
	public String toString()
	{
		return "VSite ::: name=" + getName() + ", virtualuri=" + getVirtualURI() + ", " + "realuri="+getRealURI().toString()+", inetaddress="+getInetAddress();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + ((realURI == null) ? 0 : realURI.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		VSite other = (VSite) obj;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (realURI == null) {
			if (other.realURI != null)
				return false;
		} else if (!realURI.equals(other.realURI))
			return false;
		return true;
	}

	

}
